#include "s3e.h"

extern "C" {
	#include "lualib.h"
	#include "lauxlib.h"
	#include "lua.h"
}

#include "Box2D/Box2D.h"
#include "LuaWrapper.h"

#include "GameObject.h"
#include "GameObjectLua.h"

namespace SC {

	GameObjectLua::GameObjectLua()
	{
		lua_State * L = LuaWrapper::GetPtr()->getLuaState();
		sprintf(mLuaObjName, "%x_Robot1", this);
		
		// Create an instance in Lua for Robot1
		char code[100];
		sprintf(code, "%s = Robot1:new()", mLuaObjName);
		int res = luaL_dostring(L, code);
		if (res != 0) {
			// TODO: handle error
		}
		
		// Add functions to table
		lua_getglobal(L, mLuaObjName);
		
		int i=0;
		while (GameObjectLib[i].name) {
			lua_pushstring(L, GameObjectLib[i].name);
			lua_pushcfunction(L, GameObjectLib[i].func);
			lua_settable(L, -3);
			i++;
		}

		//////////////////////
		// Create user data //
		//////////////////////

		// Associate this instance
		lua_pushstring(L, "ptr");
		GameObjectLua **obj = (GameObjectLua **)lua_newuserdata(L, sizeof(GameObjectLua *));
		*obj = this;
		lua_settable(L, -3);		

		// Clean table name
		lua_settop(L, 1);
	}

	GameObjectLua::~GameObjectLua()
	{
		
	}

	void GameObjectLua::update(uint64 delta)
	{
		// Call base class update
		GameObject::update(delta);

		// Call Lua implementation
		lua_State * L = LuaWrapper::GetPtr()->getLuaState();
		
		lua_getglobal(L, mLuaObjName);
		luaL_callmeta(L, -1, "Update");
	}
	
	// Asset State
	int GameObjectLua::GOLua_SetState(lua_State * L)
	{
		if (lua_gettop(L) != 2) {
			// TODO: report error
			return (0);
		}

		if (!lua_isuserdata(L, 1)) {
			// TODO: report error
			return (0);
		}

		if (!lua_isnumber(L, 2)) {
			// TODO: report error
			return (0);
		}

		GameObjectLua * curObject = *(GameObjectLua **)lua_touserdata(L, 1);
		
		int32 state = lua_tointeger(L, 2);
		curObject->setState(state);

		return (0);
	}

	int GameObjectLua::GOLua_GetState(lua_State * L)
	{
		if (lua_gettop(L) != 1) {
			// TODO: report error
			return (0);
		}

		if (!lua_isuserdata(L, 1)) {
			// TODO: report error
			return (0);
		}

		GameObjectLua * curObject = *(GameObjectLua **)lua_touserdata(L, 1);

		int32 state = curObject->getState();
		lua_pushinteger(L, state);

		return (1);
	}

	// Some physics
	int GameObjectLua::GOLua_GetLinearVelocity(lua_State * L)
	{
		if (lua_gettop(L) != 1) {
			// TODO: report error
			return (0);
		}

		if (!lua_isuserdata(L, 1)) {
			// TODO: report error
			return (0);
		}
		
		GameObjectLua * curObject = *(GameObjectLua **)lua_touserdata(L, 1);

		b2Vec2 lv = curObject->getLinearVelocity();
		lua_pushnumber(L, lv.x);
		lua_pushnumber(L, lv.y);

		return (2);
	}

	int GameObjectLua::GOLua_SetLinearVelocity(lua_State * L)
	{
		if (lua_gettop(L) != 3) {
			// TODO: report error
			return (0);
		}

		if (!lua_isuserdata(L, 1)) {
			// TODO: report error
			return (0);
		}

		if ( (!lua_isnumber(L, 2)) || (!lua_isnumber(L, 3)) ) {
			// TODO: report error
			return (0);
		}

		GameObjectLua * curObject = *(GameObjectLua **)lua_touserdata(L, 1);

		b2Vec2 lv;
		lv.x = (float32) lua_tonumber(L, 2);
		lv.y = (float32) lua_tonumber(L, 3);

		curObject->setLinearVelocity(lv);

		return (0);
	}

	int GameObjectLua::GOLua_ApplyForce(lua_State * L)
	{
		if (lua_gettop(L) != 3) {
			// TODO: report error
			return (0);
		}

		if (!lua_isuserdata(L, 1)) {
			// TODO: report error
			return (0);
		}

		if ( (!lua_isnumber(L, 2)) || (!lua_isnumber(L, 3)) ) {
			// TODO: report error
			return (0);
		}

		GameObjectLua * curObject = *(GameObjectLua **)lua_touserdata(L, 1);

		b2Vec2 force;
		force.x = (float32) lua_tonumber(L, 2);
		force.y = (float32) lua_tonumber(L, 3);

		curObject->applyForce(force);

		return (0);
	}

	int GameObjectLua::GOLua_GetPosition(lua_State * L)
	{
		if (lua_gettop(L) != 1) {
			// TODO: report error
			return (0);
		}

		if (!lua_isuserdata(L, 1)) {
			// TODO: report error
			return (0);
		}

		GameObjectLua * curObject = *(GameObjectLua **)lua_touserdata(L, 1);

		b2Vec2 pos = curObject->getPosition();

		lua_pushnumber(L, pos.x);
		lua_pushnumber(L, pos.y);

		return (2);
	}

	int GameObjectLua::GOLua_SetPosition(lua_State * L)
	{
		if (lua_gettop(L) != 3) {
			// TODO: report error
			return (0);
		}

		if (!lua_isuserdata(L, 1)) {
			// TODO: report error
			return (0);
		}

		if ( (!lua_isnumber(L, 2)) || (!lua_isnumber(L, 3)) ) {
			// TODO: report error
			return (0);
		}

		GameObjectLua * curObject = *(GameObjectLua **)lua_touserdata(L, 1);

		b2Vec2 pos;
		pos.x = (float32) lua_tonumber(L, 2);
		pos.y = (float32) lua_tonumber(L, 3);

		curObject->setPosition(pos);

		return (0);
	}

} // namespace SC

